/*
 * For the indicator to work, place the file
 * SmoothAlgorithms.mqh
 * in the directory: MetaTrader\\MQL5\Include
 */
//+X================================================================X+ 
//|                                              4C_Momentum_AMA.mq5 | 
//|                             Copyright  2010,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+X================================================================X+ 
#property copyright "Copyright  2010, Nikolay Kositsin"
#property link "farria@mail.redcom.ru"
//---- number of version of the indicator
#property version   "1.00"
//---- drawing the indicator in a separate window
#property indicator_separate_window 
//---- number of indicator buffers
#property indicator_buffers 5 
//+-----------------------------------+
//|  parameters of indicator drawing  |
//+-----------------------------------+
//---- only two plots are used
#property indicator_plots   2
//---- drawing indicator as a color histogram
#property indicator_type1   DRAW_COLOR_HISTOGRAM
#property indicator_type2   DRAW_COLOR_HISTOGRAM
#property indicator_color1  Blue, Green
#property indicator_color2  Magenta, Red
#property indicator_width1  2
#property indicator_width2  2
#property indicator_label1  "Momentum_AMA Upper"
#property indicator_label2  "Momentum_AMA Lower"
//+-----------------------------------+
//|  Input parameters of the indicator|
//+-----------------------------------+
enum Applied_price_ //Type of constant
  {
   PRICE_CLOSE_ = 1,     //PRICE_CLOSE
   PRICE_OPEN_,          //PRICE_OPEN
   PRICE_HIGH_,          //PRICE_HIGH
   PRICE_LOW_,           //PRICE_LOW
   PRICE_MEDIAN_,        //PRICE_MEDIAN
   PRICE_TYPICAL_,       //PRICE_TYPICAL
   PRICE_WEIGHTED_,      //PRICE_WEIGHTED
   PRICE_SIMPL_,         //PRICE_SIMPL_
   PRICE_QUARTER_,       //PRICE_QUARTER_
   PRICE_TRENDFOLLOW0_, //PRICE_TRENDFOLLOW0_
   PRICE_TRENDFOLLOW1_  //PRICE_TRENDFOLLOW1_
  };
input int Length=8; // period of Momentum
input int ama_period = 9; // period of AMA
input int fast_ma_period = 2; // period of fast MA
input int slow_ma_period = 30; // period of slow MA
input Applied_price_ IPC=PRICE_CLOSE_;//price constant
/* , used for calculation of the indicator ( 1-CLOSE, 2-OPEN, 3-HIGH, 4-LOW, 
  5-MEDIAN, 6-TYPICAL, 7-WEIGHTED, 8-SIMPL, 9-QUARTER, 10-TRENDFOLLOW, 11-0.5 * TRENDFOLLOW.) */
input double G = 2.0; // a power the smoothing constant is raised to
input int Shift=0; // horizontal shift of the indicator in bars
//---+
//---- indicator buffers
double Momentum[];
double UpColorsBuffer[];
double LoColorsBuffer[];
double UpperBuffer[];
double LowerBuffer[];

int start;
//+X================================================================X+
// Description of the function iPriceSeries                          |
// Description of the function iPriceSeriesAlert                     |
// Description of the class CMomentum                                |
//+X================================================================X+ 
#include <SmoothAlgorithms.mqh>  
//+X================================================================X+    
//| Momentum indicator initialization function                       | 
//+X================================================================X+  
void OnInit()
  {
//----+  
   start = Length+ama_period+2;
//---- turning a dynamic array into an indicator buffer
   SetIndexBuffer(4, Momentum, INDICATOR_CALCULATIONS);
//---- shifting the indicator horizontally
   PlotIndexSetInteger(4,PLOT_SHIFT, Shift);
//---- performing the shift of beginning of indicator drawing
   PlotIndexSetInteger(4, PLOT_DRAW_BEGIN, Length+ama_period+2);
//--- creation of a label to be displayed in the Data Window
   PlotIndexSetString(4, PLOT_LABEL, "Momentum_AMA");
//---- setting values of the indicator that won't be visible on the chart
   PlotIndexSetDouble(4,PLOT_EMPTY_VALUE,EMPTY_VALUE);
//----   
   SetIndexBuffer(0, UpperBuffer, INDICATOR_DATA);
   PlotIndexSetInteger(0, PLOT_SHIFT, Shift);
   PlotIndexSetInteger(0, PLOT_DRAW_BEGIN, Length+ama_period+2);
   SetIndexBuffer(1, UpColorsBuffer, INDICATOR_COLOR_INDEX);
   PlotIndexSetInteger(1, PLOT_SHIFT, Shift);
   PlotIndexSetInteger(1, PLOT_DRAW_BEGIN, Length+ama_period+2);
   SetIndexBuffer(2, LowerBuffer, INDICATOR_DATA);
   PlotIndexSetInteger(2, PLOT_SHIFT, Shift);
   PlotIndexSetInteger(2, PLOT_DRAW_BEGIN, Length+ama_period+2);
   SetIndexBuffer(3, LoColorsBuffer, INDICATOR_COLOR_INDEX);
   PlotIndexSetInteger(3, PLOT_SHIFT, Shift);
   PlotIndexSetInteger(3, PLOT_DRAW_BEGIN, Length+ama_period+2);
   
//---- initialization of a variable for a short name of the indicator
   string shortname;
   StringConcatenate(shortname,"Momentum_AMA( Length = ",Length,")");
//--- creation of the name to be displayed in a separate sub-window and in a pop up help
   IndicatorSetString(INDICATOR_SHORTNAME, shortname);
//--- determination of accuracy of displaying of the indicator values
   IndicatorSetInteger(INDICATOR_DIGITS, 0);
//---- declaration of variable of the class CJurX from the file SmoothAlgorithms.mqh
   CMomentum Mom;
//---- setting up alerts for unacceptable values of external variables
   Mom.MALengthCheck("Length", Length);
   Mom.MALengthCheck("ama_period", ama_period);
   Mom.MALengthCheck("fast_ma_period", fast_ma_period);
   Mom.MALengthCheck("slow_ma_period", slow_ma_period);
//----+ end of initialization
  }
//+X================================================================X+  
//| Momentum iteration function                                      | 
//+X================================================================X+  
int OnCalculate(
                const int rates_total,    // amount of history in bars at the current tick
                const int prev_calculated,// amount of history in bars at the previous tick
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]
                )
  {
//----+   
//---- checking the number of bars to be enough for the calculation
   if(rates_total < start) return(0);

//---- declaration of variables with a floating point  
   double price, momentum, amomentum;
//----+ declaration of integer variables and getting calculated bars
   int first, bar;

//---- calculation of the starting number 'first' for the cycle of recalculation of bars
   if(prev_calculated == 0) // checking if it's the first start of calculation of the indicator
      first = 0; // start number for calculation of all bars
   else first = prev_calculated - 1; // starting number for calculation of new bars

//---- declaration of variables of the classes CMomentum and CAMA from the file SmoothAlgorithms.mqh
   static CMomentum Mom;
   static CAMA AMA;

//---- Main cycle of calculation of the indicator
   for(bar=first; bar<rates_total; bar++)
     {
      //----+ Call of the function PriceSeries to get incrementation of the input price dprice_
      price = PriceSeries(IPC, bar, open, low, high, close);

      //----+ Two calls of the function MomentumSeries.  
      momentum = Mom.MomentumSeries(0, prev_calculated, rates_total, Length, price, bar, false);
      
      amomentum= AMA.AMASeries(Length+1, prev_calculated, rates_total,
                           ama_period, fast_ma_period, slow_ma_period, G, momentum, bar, false);

      //---- Loading the obtained value in the indicator buffer
      amomentum /=  _Point;
      Momentum[bar] = amomentum;
      //----
      UpperBuffer[bar]=0.0;
      UpColorsBuffer[bar]=0.0;
      LowerBuffer[bar]=0.0;
      LoColorsBuffer[bar]=0.0;
      //----
      if (bar <= start) continue;
      
      if (amomentum > 0)
       {
        UpperBuffer[bar] = amomentum;
        if (amomentum > Momentum[bar-1]) UpColorsBuffer[bar] = 1;
       }
      else
       {
        LowerBuffer[bar] = amomentum;
        if (amomentum < Momentum[bar-1]) LoColorsBuffer[bar] = 1;
       }

     }
//----+     
   return(rates_total);
  }
//+X----------------------+ <<< The End >>> +-----------------------X+
